﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Windows.Controls
Imports Microsoft.Practices.Prism.Commands
Imports Microsoft.Practices.Prism.Events
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Events
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Logger
Imports MsdnDemo.Infrastructure.Portable.MvpVmBase
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.Phone.Events

Namespace MsdnDemo.Base
    Public Class PresentationPresenterBase(Of TView As IView, TViewModel As IMvpVmViewModel)
        Inherits MvpVmPresenter(Of TView, TViewModel)
#Region "Fields "
        Private _logger As ILogger
        Private _eventAggregator As IEventAggregator
        Private ReadOnly _queuedViewEvents As New List(Of ViewEventArgs)()
#End Region

        <Dependency()>
        Public Property Logger() As ILogger
            Get
                Return _logger
            End Get
            Set(ByVal value As ILogger)
                _logger = value
                TryInitialize()
            End Set
        End Property

        <Dependency()>
        Public Shadows Property Shell() As IShell
            Get
                Return MyBase.Shell
            End Get
            Set(ByVal value As IShell)
                MyBase.Shell = value
                TryInitialize()
            End Set
        End Property

        <Dependency()>
        Public Shadows Property View() As TView
            Get
                Return MyBase.View
            End Get
            Set(ByVal value As TView)
                MyBase.View = value

                ViewPropertySet()

                TryInitialize()
            End Set
        End Property

        <Dependency()>
        Public Property EventAggregator() As IEventAggregator
            Get
                Return _eventAggregator
            End Get
            Set(ByVal value As IEventAggregator)
                _eventAggregator = value

                ' If view events were fired while the EventAggregator
                ' was not yet instantiated they will be queued.  We'll
                ' need to publish them
                For Each viewEvent In _queuedViewEvents
                    EventAggregator.GetEvent(Of ViewEvent)().Publish(viewEvent)
                Next viewEvent

                _queuedViewEvents.Clear()

                ' Subscribe to message events
                EventAggregator.GetEvent(Of MessageEvent)().Subscribe(AddressOf MessageEventHandler)

                SubscribeToDataEvent(AddressOf HandleDataArrivedEvent)
            End Set
        End Property

        <Dependency()>
        Public Shadows Property ViewModel() As TViewModel
            Get
                Return MyBase.ViewModel
            End Get
            Set(ByVal value As TViewModel)
                MyBase.ViewModel = value
                Dim _viewModel = TryCast(value, PresentationViewModelBase)

                ' Wire up our button command which the ButtonStyle
                ' has a Command binding to
                _viewModel.ButtonCommand = New DelegateCommand(Of Object)(AddressOf ExecuteButtonCommandHandler, AddressOf CanExecuteButtonCommandHandler)

                _viewModel.Presenter = Me

                TryInitialize()
            End Set
        End Property

        Private Sub TryInitialize()
            If View Is Nothing OrElse ViewModel Is Nothing OrElse Shell Is Nothing OrElse Logger Is Nothing Then
                Return
            End If

            View.DataContext = ViewModel

            ' Hook for subclass so that it can initializes 
            ' with all dependency properties set
            Initialize()
        End Sub

        Protected Overridable Sub Initialize()
            Logger.Log("{0}.Initialize()", Me.GetType())
        End Sub


        Private Function CanExecuteButtonCommandHandler(ByVal e As Object) As Boolean
            Dim args = New DelegateCommandEventArgs With {.Sender = e}
            Return CanExecuteButtonCommandHandler(Me, args)
        End Function

        Protected Overridable Function CanExecuteButtonCommandHandler(ByVal sender As Object, ByVal e As DelegateCommandEventArgs) As Boolean
            Return True
        End Function

        Private Sub ExecuteButtonCommandHandler(ByVal e As Object)
            Dim args As DelegateCommandEventArgs
            If TypeOf e Is DelegateCommandEventArgs Then
                args = TryCast(e, DelegateCommandEventArgs)
            Else
                args = New DelegateCommandEventArgs With {.Sender = e}
            End If

            ' Give virtual method an opportunity to validate
            Logger.Log("{0}", args)

            If CanExecuteButtonCommandHandler(Me, args) Then
                ExecuteButtonCommandHandler(Me, args)
            End If
        End Sub

        Protected Overridable Sub ExecuteButtonCommandHandler(ByVal sender As Object, ByVal e As DelegateCommandEventArgs)
            Dim message As String = Nothing
            If e.ViewModel Is Nothing Then
                Dim button = TryCast(e.Sender, Button)
                message = String.Format("{0}", e.Sender)
            Else
                message = String.Format("OVERRIDE ExecuteButtonCommandHandler" & vbCrLf & "Sender: {0}" & vbCrLf & "VM: {1}" & vbCrLf & "ID: {2}", e.Sender, e.ViewModel.GetType().Name, e.Tag.ToString())

            End If
            Logger.Log(message)
        End Sub

        Public Overrides Sub ViewActivated(ByVal e As ViewEventArgs)
            Logger.Log("{0} ViewActivated", Me.GetType().Name)
        End Sub

        Private Sub ViewPropertySet()
            ' Event aggregator will not be available so we'll queue
            ' it until EventAggregator set.
            Dim viewEvent = New ViewEventArgs With {.View = View, .ProcessType = ProcessType.NewView}

            ' Process or queue event
            If EventAggregator IsNot Nothing Then
                EventAggregator.GetEvent(Of ViewEvent)().Publish(viewEvent)
            Else
                _queuedViewEvents.Add(viewEvent)
            End If
        End Sub

        ''' <summary>
        ''' Subscribes to data event.
        ''' </summary>
        ''' <param name="callback">The callback.</param>
        Protected Sub SubscribeToDataEvent(ByVal callback As Action(Of DataEventArgs))
            EventAggregator.GetEvent(Of DataEvent)().Subscribe(callback)
        End Sub


        ''' <summary>
        ''' Handles the data arrived event.   
        ''' Has to be public method since it handles event
        ''' subscription.
        ''' </summary>
        ''' <param name="e">The <see cref="DataEventArgs"/> 
        ''' instance containing the event data.</param>
        Public Overridable Sub HandleDataArrivedEvent(ByVal e As DataEventArgs)
            Try
                Select Case e.ProcessType
                    Case ProcessType.DataCreate, ProcessType.DataDelete, ProcessType.DataList, ProcessType.DataRead, ProcessType.DataUpdate
                End Select
            Catch ex As Exception
                Logger.Log("{0}:MvpVmPresenter.HandleDataArrivedEvent({1}): " & vbCrLf & vbTab & "{2}{3}", Me.GetType().Name, e.ProcessType, ex.Message, ex.StackTrace)
            End Try
        End Sub

        ''' <summary>
        ''' Message event handler.
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.MessageEventArgs"/> instance containing the event data.</param>
        Public Overridable Sub MessageEventHandler(ByVal e As MessageEventArgs)
            Logger.Log("MESSAGE EVENT:{0} IsError:{1} Status:{2} ({3})", e.Message, e.IsError, e.StatusCode, Me.GetType().Name)
        End Sub


        Protected Sub RaiseMessageEvent(ByVal message As String)
            EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Message = message, .Type = ProcessType.Message, .Sender = Me})
        End Sub

        ''' <summary>
        ''' Raises the view event.
        ''' </summary>
        ''' <param name="moduleName">Name of the module.</param>
        ''' <param name="viewName">Name of the view.</param>
        ''' <param name="processType">Type of the process.</param>
        Protected Sub RaiseViewEvent(ByVal moduleName As String, ByVal viewName As String, ByVal processType As ProcessType)
            EventAggregator.GetEvent(Of ViewEvent)().Publish(New ViewEventArgs With {.ModuleName = moduleName, .ProcessType = ProcessType.ActivateView, .ViewNameToActivate = viewName})
        End Sub

        ''' <summary>
        ''' Notify the ApplicationController that the login was successful so that it can
        ''' load (on demand) the password manager module (as configured in the Bootstrapper)
        ''' Once modules are loaded they will not be loaded again so it is harmless on
        ''' successive calls.
        ''' </summary>
        ''' <param name="moduleName">Name of the module.</param>
        ''' <param name="viewName">Name of the view.</param>
        ''' <param name="eventType">Type of the event.</param>
        Protected Sub RaiseModuleEvent(ByVal moduleName As String, ByVal viewName As String, ByVal eventType As ModuleEventType)
            EventAggregator.GetEvent(Of ModuleEvent)().Publish(New ModuleEventArgs With {.ModuleName = moduleName, .EventType = eventType, .PayLoad = viewName}) ' The view to load on click
        End Sub
    End Class
End Namespace