﻿Imports System
Imports Microsoft.Practices.Prism.Events
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Events
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Logger
Imports MsdnDemo.Infrastructure.Portable.Types

Namespace MsdnDemo.LayerBusinessLogic.Base
    Public Class BllBase(Of TEntity)
        Implements IBll
        Public Property IsFailRequiredForUnitTest() As Boolean Implements IBll.IsFailRequiredForUnitTest

        <Dependency()>
        Public Property Dal() As IDal(Of TEntity)

        <Dependency()>
        Public Property Entity() As TEntity

        <Dependency()>
        Public Property EventAggregator() As IEventAggregator

        <Dependency()>
        Public Property Logger() As ILogger

        Public Overridable Sub Initialize() Implements IBll.Initialize
        End Sub

       

        ''' <summary>
        ''' Creates the table.
        ''' </summary>
        ''' <returns></returns>
        Public Function CreateTable() As Integer Implements IBll.CreateTable
            Try
                Dim result = Dal.CreateTable(Me, Nothing)

                If IsFailRequiredForUnitTest Then
                    Throw New Exception(Messages.UnitTestBogusErrorMessage)
                End If

                Return result
            Catch ex As Exception
                ' See if the error was because the table already exists
                Dim statusCodeDemo = StatusCode.NotDefined
                If ex.Message.ToLower().Contains(Messages.AlreadyExists) Then
                    statusCodeDemo = StatusCode.AlreadyExists
                End If

                ' Publish message event
                EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Exception = ex, .Message = String.Format(Messages.CouldNotCreateTableP0, GetType(TEntity).Name), .Type = ProcessType.CreateTable, .ErrorCode = StatusCode.TableCreateError, .StatusCode = statusCodeDemo})

                Return StatusCode.TableCreateError
            End Try
        End Function

        ''' <summary>
        ''' Drops the table.
        ''' </summary>
        ''' <returns></returns>
        Public Function DropTable() As Integer Implements IBll.DropTable
            Try
                Dim result = Dal.DropTable(Me, Nothing)

                If IsFailRequiredForUnitTest Then
                    Throw New Exception(Messages.UnitTestBogusErrorMessage)
                End If

                Return result
            Catch ex As Exception
                ' Publish message event
                EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Exception = ex, .Message = String.Format(Messages.CouldNotDropTableP0, GetType(TEntity).Name), .Type = ProcessType.CreateTable, .ErrorCode = StatusCode.TableDropError, .StatusCode = StatusCode.NotDefined})

                Return StatusCode.TableCreateError
            End Try
        End Function

        ''' <summary>
        ''' Creates the specified record.  Expects e.Data
        ''' to be populated with record to create
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Function Create(ByVal e As DalEventArgs) As Integer Implements IBll.Create
            e.Type = ProcessType.DataCreate
            Dim result = Create(Me, e)

            If result = StatusCode.DoesNotExist Then
                result = CreateTableWhenNotExist(e)
                If result = StatusCode.Success Then
                    Return Create(e)
                End If
            End If
            Return result
        End Function

        Private Function Create(ByVal sender As Object, ByVal e As DalEventArgs) As Integer
            e.Type = ProcessType.DataCreate
            Try
                Dim result = Dal.Create(Me, e)

                ' Store the primary key returned by
                ' data access layer
                e.PrimaryKey = result

                If e.Exception IsNot Nothing Then
                    If e.Exception.Message.Contains("column id is not unique") Then
                        Return StatusCode.UniquenessConstraint
                    End If

                    Return StatusCode.RecordCreateError
                End If

                Return StatusCode.Success
            Catch ex As Exception
                If ex.Message.Contains("DEVELOPER") Then
                    Throw ex
                End If

                If ex.Message.Contains(Messages.NoSuchTable) Then
                    Return StatusCode.DoesNotExist
                End If

                ' Publish message event
                EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Exception = ex, .Message = String.Format(Messages.CouldNotCreateRecordP0, GetType(TEntity).Name), .Type = ProcessType.DataCreate, .ErrorCode = StatusCode.RecordCreateError, .StatusCode = StatusCode.NotDefined})

                Return StatusCode.TableCreateError
            End Try
        End Function

        ''' <summary>
        ''' Reads the record specified by e.PrimaryKey
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Function Read(ByVal e As DalEventArgs) As Integer Implements IBll.Read
            e.Type = ProcessType.DataRead
            e.Results = Dal.Read(Me, e)

            If e.Results Is Nothing Then
                Return StatusCode.NoData
            End If

            Return StatusCode.Success
        End Function

        ''' <summary>
        ''' Updates the specified e.
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Function Update(ByVal e As DalEventArgs) As Integer Implements IBll.Update
            e.Type = ProcessType.DataUpdate
            Dim status = Dal.Update(Me, e)

            Return status
        End Function

        ''' <summary>
        ''' Deletes the specified record
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Function Delete(ByVal e As DalEventArgs) As Integer Implements IBll.Delete
            e.Type = ProcessType.DataDelete
            Dim status = Dal.Delete(Me, e)

            Return status
        End Function

         
        ''' <summary>
        ''' Lists the specified e.
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Function List(ByVal e As DalEventArgs) As Integer Implements IBll.List
            Dim isError = False

tryAgain:
            Try
                e.Results = Dal.ReadList(Me, e)
            Catch ex As Exception
                If (Not isError) AndAlso ex.Message.Contains(Messages.NoSuchTable) Then
                    isError = True
                    Dim status = CreateTableWhenNotExist(e)
                    If status <> StatusCode.Success Then
                        Return StatusCode.TableCreateError
                    End If
                    GoTo tryAgain
                End If
                Throw
            End Try
            ' Shouldn't be null
            If e.Results Is Nothing Then
                Return StatusCode.NoData
            End If

            ListDataArrivedHandler(e)

            Return StatusCode.Success
        End Function

        Protected Overridable Function CreateTableWhenNotExist(ByVal e As DalEventArgs) As Integer
            Return CreateTable()
        End Function

        Protected Overridable Sub ListDataArrivedHandler(ByVal e As DalEventArgs)
            Logger.Log("VIRTUAL: {0}.ListDataArrived()", Me.GetType().Name)
        End Sub

    End Class
End Namespace