﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Events
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Entities
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.LayerBusinessLogic.Base
Imports MsdnDemo.LayerBusinessLogic.Interfaces

Namespace MsdnDemo.LayerBusinessLogic
    Public Class UserBll
        Inherits BllBase(Of UserEntity)
        Implements IUserBll
        Private _isProcessing As Boolean

        <Dependency()>
        Public Property RoleBll() As IRolesBll

        <Dependency()>
        Public Property UserRoleBll() As IUserRoleBll

        <Dependency()>
        Public Property SecurityViewModel() As ISecurityViewModel

        Public Property Roles() As List(Of RoleEntity)

        Public Overrides Sub Initialize()
            ' Get the system roles
            Dim roleResults = New DalEventArgs
            Dim status = RoleBll.List(roleResults)
            If status <> StatusCode.Success Then
                Throw New Exception("Could not get system roles!", roleResults.Exception)
            End If

            ' Get a list of roles supported by system.  We'll
            ' use this is AuthenticateUser to join with the 
            ' user roles (to get role name)
            Roles = roleResults.GetResults(Of IEnumerable(Of RoleEntity))().ToList()
        End Sub

        Private Function GetUserRoles() As IEnumerable(Of UserRoleEntity)
            ' Get user roles from user/role xref table
            Dim userRoleResults = New DalEventArgs With {.WhereClause = String.Format("UserFk={0}", SecurityViewModel.Id)}

            Dim status = UserRoleBll.List(userRoleResults)
            If status <> StatusCode.Success Then
                Throw New Exception("Could not get user roles!", userRoleResults.Exception)
            End If

            ' Get the user roles from the above List() 
            Dim userRoles = userRoleResults.GetResults(Of IEnumerable(Of UserRoleEntity))()

            Return userRoles
        End Function

        Public Function DropRoleFromUser(ByVal roleList As String) As Integer Implements IUserBll.DropRoleFromUser
            Dim status = StatusCode.Success

            Dim userRoles = GetUserRoles().ToList()

            For Each role In roleList.Split(","c)

                Dim roleVar = role
                Dim systemRole = Roles.First(Function(r)
                                                 Return Object.Equals(r.Role, roleVar)
                                             End Function)
                If systemRole Is Nothing Then ' Invalid role
                    Continue For
                End If

                Dim roleId = systemRole.Id
                Dim userRoleId = userRoles.FirstOrDefault(Function(u) u.RoleFk = roleId)
                If userRoleId Is Nothing Then ' User does not have role
                    Continue For
                End If

                ' Prepare to delete the role from the user
                Dim args = New DalEventArgs With {.PrimaryKey = userRoleId.Id}
                status = UserRoleBll.Delete(args)
                If status <> StatusCode.Success Then
                    Logger.Log("Could not delete role {0} " & vbCrLf & "{1}{2}", roleVar, args.Exception.Message, args.Exception.StackTrace)
                End If
            Next


                ' Get the user roles from the user role table
                SecurityViewModel.Roles = (
                    From u In GetUserRoles()
                    Join r In Roles On u.RoleFk Equals r.Id
                    Select r.Role).ToList()

                Return status
        End Function


        Public Function CreateUsersWithRoles(ByVal userEntity As UserEntity, ByVal roleList As String) As Integer Implements IUserBll.CreateUsersWithRoles
            Dim roles = roleList.Split(","c).Select(Function(role) New RoleEntity With {.Role = role}).ToList()

            Return CreateUsersWithRoles(userEntity, roles)
        End Function

        Public Function CreateUsersWithRoles(ByVal userEntity As UserEntity, ByVal roleList As List(Of RoleEntity)) As Integer Implements IUserBll.CreateUsersWithRoles
            Dim userResult = New DalEventArgs With {.Data = userEntity}
            Dim statusDemo = Create(userResult)
            If statusDemo <> StatusCode.Success Then
                Throw New Exception("Could not create user with roles", userResult.Exception)
            End If

            Dim userRecord = userResult.GetResults(Of UserEntity)()
            userRecord.Id = userResult.PrimaryKey

            ' Build UserRoleEntity records
            Dim userRoles = From r In Roles
                            Join l In roleList On r.Role Equals l.Role
                            Select New UserRoleEntity With {.RoleFk = r.Id, .UserFk = userRecord.Id}

            ' Create UserRoleEntity records in table
            For Each role In userRoles
                UserRoleBll.Create(New DalEventArgs With {.Data = role})
            Next role

            Return StatusCode.Success
        End Function


        Public Function AuthenticateUser(ByVal password As String) As Boolean Implements IUserBll.AuthenticateUser
            Dim returnArgs = New DalEventArgs With {.WhereClause = String.Format("PrimaryEmail='{0}'", SecurityViewModel.Login)}
            ' Dal is responsible for processing the where clause
            ' as applicable - we only want records where the 
            ' primary email (could be multiple accounts) is equal
            ' to the login

            ' Get the list of users that have a PrimaryEmail=Login
            Dim status = List(returnArgs)

            ' If there was an exception (not failed login)
            If status <> StatusCode.Success Then
                Throw New Exception("Could not authenticate!", returnArgs.Exception)
            End If

            ' Get the user list from the results
            Dim userList = returnArgs.GetResults(Of IEnumerable(Of UserEntity))()

            ' If multiple accounts we want to ensure we check all of them
            ' against provided password
            Dim user = userList.FirstOrDefault(Function(u) u.PrimaryEmail = SecurityViewModel.Login AndAlso u.Password = password)

            ' Set flag
            SecurityViewModel.IsAuthenticated = user IsNot Nothing

            If SecurityViewModel.IsAuthenticated Then
                SecurityViewModel.Id = user.Id

                Dim userRoles = GetUserRoles()

                ' Get the user roles from the user role table
                SecurityViewModel.Roles = (
                    From u In userRoles
                    Join r In Roles On u.RoleFk Equals r.Id
                    Select r.Role).ToList()

            End If
            Return SecurityViewModel.IsAuthenticated
        End Function


        Protected Overrides Function CreateTableWhenNotExist(ByVal e As DalEventArgs) As Integer
            Dim statusDemo = MyBase.CreateTableWhenNotExist(e)
            If statusDemo = StatusCode.Success Then
                statusDemo = CreateDefaultUser(e)
            End If
            Return statusDemo
        End Function


        Private Function CreateDefaultUser(ByVal e As DalEventArgs) As Integer
            CreateUsersWithRoles(New UserEntity With {.Id = 1, .FirstName = "System", .LastName = "Administrator", .PrimaryEmail = "admin@mydomain.net", .Password = "password"}, "Admin,User")

            CreateUsersWithRoles(New UserEntity With {.FirstName = "Jane", .LastName = "Doe", .Password = "password", .PrimaryEmail = "jane@mydomain.net"}, "User")

            CreateUsersWithRoles(New UserEntity With {.FirstName = "Guest", .LastName = "User", .Password = "password", .PrimaryEmail = "guest@mydomain.net"}, "Guest")

            Return StatusCode.Success
        End Function


        Public Function GetUserList() As List(Of UserEntity) Implements IUserBll.GetUserList
            Dim userArgs = New DalEventArgs

            ' Request the user list
            Dim status = List(userArgs)

            ' If not successful than raise event for subscribers
            If status <> StatusCode.Success Then
                EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Exception = userArgs.Exception, .Message = "Could not get User List!", .ErrorCode = StatusCode.ListFailed, .IsError = True})
                Return New List(Of UserEntity)()
            End If

            ' Get the list from the arguments
            Dim users = userArgs.GetResults(Of List(Of UserEntity))()

            EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Message = String.Format("{0} user records", users.Count)})
            Return users
        End Function

        
    End Class
End Namespace