﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports Community.CsharpSqlite.Gwn.Library.SqliteDataProvider
Imports Community.CsharpSqlite.Gwn.Library.SqliteDataProvider.Base
Imports Community.CsharpSqlite.Gwn.Library.SqliteDataProvider.Exceptions
Imports Microsoft.Practices.Prism.Logging
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Logger
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.LayerBusinessLogic.Interfaces
Imports MsdnDemo.LayerDataAccess.Sqlite

Namespace MsdnDemo.LayerDataAccess.Sql
    ''' <summary>
    ''' Data Access Layer base class
    ''' </summary>
    ''' <typeparam name="TEntity">The type of the entity.</typeparam>
    Public Class SqlDal(Of TEntity)
        Implements IDal(Of TEntity)
        <Dependency()>
        Public Property Logger() As ILogger

        <Dependency()>
        Public Property SecurityViewModel As ISecurityViewModel

        Public Property UserBll As IUserBll

        ''' <summary>
        ''' Occurs when [data arrived event].
        ''' </summary>
        Public Event DataArrivedReadEvent As EventHandler(Of DataEventArgs) Implements IDal(Of TEntity).DataArrivedReadEvent

        ''' <summary>
        ''' Gets or sets the connection string.
        ''' </summary>
        ''' <value>The connection string.</value>

        Public Property ConnectionString As String Implements IDal(Of TEntity).ConnectionString


        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is default data used during create.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is default data used during create; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsDefaultDataUsedDuringCreate As Boolean

        ''' <summary>
        ''' SqliteHelper reference
        ''' </summary>
        Public SqlUtility As New SqlUtility(GetType(TEntity))


        ''' <summary>
        ''' Initializes a new instance of the see cref="SqlDal&lt;TEntity&gt;"/> class.
        ''' </summary>
        Public Sub New()
            Me.New("MsdnDemo.1.0.0.5")
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the see cref="SqlDal&lt;TEntity&gt;"/> class.
        ''' </summary>
        ''' <param name="connectionString">The connection string.</param>
        Public Sub New(ByVal connectionString As String)
            Me.ConnectionString = connectionString
            IsDefaultDataUsedDuringCreate = True
        End Sub


        ''' <summary>
        ''' Creates the table.   Both CreateTable and CreateRecord
        ''' both use the same process - the DalEventArgs.Type flag 
        ''' determines which process to execute.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function CreateTable(ByVal sender As Object, ByVal e As EventArgs) As Integer Implements IDal(Of TEntity).CreateTable
            Dim args = If(TryCast(e, DalEventArgs), New DalEventArgs())
            args.Type = ProcessType.CreateTable
            Return Create(sender, args)
        End Function

        ''' <summary>
        ''' Deletes the table.   Both DeleteTable and DeleteRecord
        ''' both use the same process - the DalEventArgs.Type flag 
        ''' determines which process to execute.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function DropTable(ByVal sender As Object, ByVal e As EventArgs) As Integer Implements IDal(Of TEntity).DropTable
            Dim args = If(TryCast(e, DalEventArgs), New DalEventArgs())
            args.Type = ProcessType.DropTable
            Return Delete(sender, args)
        End Function

        ''' <summary>
        ''' Creates the specified sender.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function Create(ByVal sender As Object, ByVal e As EventArgs) As Integer Implements IDal(Of TEntity).Create
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupportedInP1, e.GetType().Name, Me.GetType().FullName))
            End If

            Dim status = -1
            Using conn = New SqliteConnection(ConnectionString)
                Try
                    conn.Open()

                    ' INSERT RECORD
                    If CType(args.Type, ProcessType) = ProcessType.DataCreate Then
                        Return CreateRecord(conn, e)
                    End If

                    ' CREATE TABLE
                    If CType(args.Type, ProcessType) = ProcessType.CreateTable Then
                        Dim sqlCreateCommand = SqlUtility.MakeSqlCreateTableString()

                        Logger.Log("Create(): " & sqlCreateCommand, Category.Debug, Priority.None)

                        Return CreateTable(conn, sqlCreateCommand)
                    End If

                Catch ex As Exception
                    Throw New SqlException(ex.Message, ex)
                End Try

                status = 0
            End Using
            Return status

        End Function



        ' CRUD Methods


        ''' <summary>
        ''' Creates the record.
        ''' </summary>
        ''' <param name="conn">The conn.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Private Function CreateRecord(ByVal conn As SqliteConnection, ByVal e As EventArgs) As Integer
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupported, e.GetType().FullName))
            End If

            ' If the primary key is -1 then it has not been initialized
            ' we'll query the table for the highest primary key so that
            ' we can auto-increment it
            If args.PrimaryKey = -1 Then
                ' TODO: see if there is a "TOP 1" syntax available
                Dim getNextKeySql = String.Format("select {0} from {1} order by {0} DESC ", args.PrimaryKeyField, GetClassName())

                Logger.Log("{0}", getNextKeySql)

                ' Execute the query to get the highest primary key
                Dim nextKeyCmd = New SqliteCommand(getNextKeySql, conn)
                Dim keyValue = nextKeyCmd.ExecuteScalar()

                ' If it came back null (saved as null) then we'll use 0
                If TypeOf keyValue Is String AndAlso keyValue.ToString() = "<null>" Then
                    keyValue = 0
                End If

                ' Set the last key
                args.PrimaryKey = Int32.Parse(keyValue.ToString())

                ' Increment it it so that we're now the highest key
                args.PrimaryKey += 1
            End If

            ' Create the insert statement
            PopulateEntityOnCreateRecord(args)
            Dim sqlCreateCommand = SqlUtility.MakeSqlInsertTableString(e)

            Logger.Log("CreateRecord(): " & sqlCreateCommand, Category.Debug, Priority.None)

            Try
                ' Execute the insert command
                Dim cmd = New SqliteCommand(sqlCreateCommand, conn)
                cmd.ExecuteScalar()
            Catch ex As Exception
                args.Exception = ex
                Logger.Log("{0}", ex.Message)
            End Try

            ' Return the primary key value used
            Return args.PrimaryKey
        End Function

        ''' <summary>
        ''' Populates the entity on create record.   The contents of
        ''' args.Data will be used to dynamically generate a Sql insert
        ''' string which will be executed for the table
        ''' </summary>
        ''' <param name="args">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        Protected Overridable Sub PopulateEntityOnCreateRecord(ByVal args As DalEventArgs)
        End Sub

        Private Shared Function GetKey(ByVal args As DalEventArgs) As Object
            If args.PrimaryKey = -999 Then
                Return String.Format("'{0}'", args.UniqueKey)
            End If
            Return args.PrimaryKey
        End Function

        ''' <summary>
        ''' Reads the specified record.  e.Data.PrimaryKey value
        ''' is expected to hold the primary key of the record
        ''' that you want to read.  If e.Data implements 
        ''' ISupportInitialize it will execute its methods
        ''' prior to, and after, populating the data instance.
        ''' Executes PopulateEntityOnRead() on applicable DAL
        ''' which is needs to be overridden to populate the
        ''' instance as applicable.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function Read(ByVal sender As Object, ByVal e As EventArgs) As TEntity Implements IDal(Of TEntity).Read
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupportedInP1, e.GetType().Name, Me.GetType().FullName))
            End If

            Using conn = New SqliteConnection(ConnectionString)
                Try
                    conn.Open()

                    Dim readSql = String.Format("select * from {0} where {1}={2}", GetClassName(), args.PrimaryKeyField, GetKey(args))

                    Logger.Log("Read(): " & readSql, Category.Debug, Priority.None)

                    ' Initialize the command
                    Dim cmd = New SqliteCommand(readSql, conn)

                    ' Execute the reader and read the first (only) record
                    Dim rdr = cmd.ExecuteReader()
                    If rdr.Read() Then
                        ' Instantiate a new instance of entity
                        Dim data = Activator.CreateInstance(Of TEntity)()

                        If TypeOf data Is ISupportInitialize Then
                            CType(data, ISupportInitialize).BeginInit()
                        End If

                        ' Let the parent populate it

                        Logger.Log("Read(): {0} populating record", Me.GetType().Name)

                        PopulateEntityOnRead(rdr, data)

                        If TypeOf data Is ISupportInitialize Then
                            CType(data, ISupportInitialize).EndInit()
                        End If

                        ' Raise conventional event
                        If args.IsEventRaised Then
                            RaiseEvent DataArrivedReadEvent(Me, New DataEventArgs With {.EventArgs = args, .Data = data, .Message = readSql, .Sender = Me, .ProcessType = ProcessType.DataRead})
                        End If


                        Return data
                    End If
                    Return Nothing
                Catch ex As Exception
                    Throw New SqlException(ex.Message)
                End Try

            End Using
        End Function


        ''' <summary>
        ''' Reads the list.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function ReadList(ByVal sender As Object, ByVal e As EventArgs) As IEnumerable(Of TEntity) Implements IDal(Of TEntity).ReadList
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupportedInP1, e.GetType().Name, Me.GetType().FullName))
            End If

            Using conn = New SqliteConnection(ConnectionString)
                Dim returnList = New List(Of TEntity)

                Try
                    conn.Open()

                    Dim isRetry = False
                    Dim lastRetryMessage = ""

RetryCommand:

                    Dim readSql = String.Format("select * from {0}", GetClassName())

                    ReadListWhereClauseSetter(args)
                    If args.WhereClause IsNot Nothing Then
                        readSql = String.Format("{0} where {1}", readSql, args.WhereClause)
                    End If

                    Logger.Log(readSql, Category.Debug, Priority.None)


                    ' Initialize the command
                    Dim cmd = New SqliteCommand(readSql, conn)

                    ' Execute the reader and read the first (only) record
                    Dim rdr = cmd.ExecuteReader()

                    Logger.Log("{0} populating record", Me.GetType().Name)
                    Do While rdr.Read()
                        ' Instantiate a new instance of entity
                        Dim data = Activator.CreateInstance(Of TEntity)()

                        If TypeOf data Is ISupportInitialize Then
                            CType(data, ISupportInitialize).BeginInit()
                        End If

                        Try
                            ' Let the parent populate it
                            PopulateEntityOnRead(rdr, data)
                        Catch ex As IndexOutOfRangeException
                            ' If we haven't already retried for this error
                            ' and the error is handled by parent class 
                            ' then retry this command
                            If (Not isRetry) AndAlso lastRetryMessage <> ex.Message AndAlso HandleIndexOutOfRangeException(ex) Then
                                lastRetryMessage = ex.Message
                                isRetry = True
                                GoTo RetryCommand
                            End If
                            Throw ' If we get here then rethrow it
                        End Try

                        If TypeOf data Is ISupportInitialize Then
                            CType(data, ISupportInitialize).EndInit()
                        End If

                        ' Add record to list
                        returnList.Add(data)
                    Loop

                    ' Raise conventional event
                    RaiseEvent DataArrivedReadEvent(Me, New DataEventArgs With {.EventArgs = args, .Data = returnList, .Message = readSql, .Sender = Me, .ProcessType = ProcessType.DataRead})

                    Return returnList
                Catch ex As Exception
                    Throw New SqlException(ex.Message)
                End Try
            End Using
        End Function

        ''' <summary>
        ''' Handles the index out of range exception.
        ''' </summary>
        ''' <param name="ex">The ex.</param>
        ''' <returns></returns>
        Protected Overridable Function HandleIndexOutOfRangeException(ByVal ex As IndexOutOfRangeException) As Boolean
            Return False
        End Function

        ''' <summary>
        ''' Hook into ReadList to obtain WhereClause
        ''' </summary>
        ''' <param name="args">The <see cref="DalEventArgs"/> instance containing the event data.</param>
        Protected Overridable Sub ReadListWhereClauseSetter(ByVal args As DalEventArgs)
        End Sub

        ''' <summary>
        ''' If the ReadList fails the parent has an opportunity
        ''' to create a new one.  If parent creates table it must
        ''' set DalEventArgs.IsTableCreated to true so that the
        ''' business logic layer will know not to attempt to create
        ''' it also
        ''' </summary>
        ''' <param name="e">The <see cref="DalEventArgs"/> instance containing the event data.</param>
        ''' <remarks></remarks>
        Public Overridable Sub TableDoesNotExistHandler(ByVal e As DalEventArgs) Implements IDal(Of TEntity).TableDoesNotExistHandler

        End Sub


        ''' <summary>
        ''' Updates the specified sender.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function Update(ByVal sender As Object, ByVal e As EventArgs) As Integer Implements IDal(Of TEntity).Update
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupportedInP1, e.GetType().Name, Me.GetType().FullName))
            End If

            Using conn = New SqliteConnection(ConnectionString)
                Try
                    conn.Open()

                    ' Create the insert statement
                    Dim sqlUpdateCommand = SqlUtility.MakeSqlUpdateTableString(e)

                    Logger.Log(sqlUpdateCommand, Category.Debug, Priority.None)

                    ' Execute the insert command
                    Dim cmd = New SqliteCommand(sqlUpdateCommand, conn)
                    cmd.ExecuteNonQuery()


                Catch ex As Exception
                    Throw New SqlException(ex.Message)
                End Try
            End Using
            Return StatusCode.Success
        End Function


        ''' <summary>
        ''' Deletes the specified sender.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Public Overridable Function Delete(ByVal sender As Object, ByVal e As EventArgs) As Integer Implements IDal(Of TEntity).Delete
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format(Messages.P0IsNotSupportedInP1, e.GetType().Name, Me.GetType().FullName))
            End If

            Using conn = New SqliteConnection(ConnectionString)
                Try
                    conn.Open()

                    ' If we're dropping the table - drop it and return
                    If CType(args.Type, ProcessType) = ProcessType.DropTable Then
                        Return DeleteTable(conn)
                    End If


                    ' Default behavior is to delete the record
                    Dim readSql = String.Format("Delete from {0} where {1}={2}", GetClassName(), args.PrimaryKeyField, args.PrimaryKey)

                    Logger.Log(readSql, Category.Debug, Priority.None)

                    ' Initialize the command
                    Dim cmd = New SqliteCommand(readSql, conn)
                    Dim status = cmd.ExecuteNonQuery()

                Catch ex As Exception
                    Throw New SqlException(ex.Message)
                End Try
            End Using
            Return StatusCode.Success
        End Function


        ' Support methods

        Private Function CreateTable(ByVal conn As SqliteConnection, ByVal sqlCreateCommand As String) As Integer
            Dim cmd = New SqliteCommand(sqlCreateCommand, conn)
            Return cmd.ExecuteNonQuery()
        End Function

        Private Function DeleteTable(ByVal conn As SqliteConnection) As Integer
            ' create the sql statement to drop the table
            Dim deleteCommandSql = String.Format("drop table {0}", GetClassName())

            Logger.Log(deleteCommandSql, Category.Debug, Priority.None)

            Try
                ' Execute the sql command
                Dim cmd = New SqliteCommand(deleteCommandSql, conn)
                cmd.ExecuteNonQuery()
            Catch ex As SqliteConnectionException
                ' If we are trying to drop a table and it doesn't
                ' exist, it is not a real error
                If ex.Message.Contains("no such table") Then
                    Return StatusCode.DoesNotExist
                End If

                ' Otherewise throw the error
                Throw
            End Try
            Return StatusCode.Success
        End Function


        ''' <summary>
        ''' Gets the name of the class.
        ''' </summary>
        ''' <returns></returns>
        Private Function GetClassName() As String
            ' We don't want ViewModel in our table names
            Return GetType(TEntity).Name.Replace("ViewModel", "")
        End Function


        ''' <summary>
        ''' Sets the connection string.
        ''' </summary>
        ''' <param name="connectionString">The connection string.</param>
        Public Overridable Sub SetConnectionString(ByVal connectionString As String) Implements IDal(Of TEntity).SetConnectionString
            Me.ConnectionString = connectionString
        End Sub


        ''' <summary>
        ''' Populates the entity when data arrives.
        ''' </summary>
        ''' <param name="rdr">The RDR.</param>
        ''' <param name="data">The data.</param>
        Protected Overridable Sub PopulateEntityOnRead(ByVal rdr As IDataReader, ByVal data As TEntity)
            Throw New NotImplementedException("You must override PopulateEntityWhenDataArrived in " & Me.GetType().Name)
        End Sub


        ''' <summary>
        ''' Updates the specified sender.
        ''' </summary>
        ''' <param name="sqlCommand">The SQL command.</param>
        ''' <returns></returns>
        Public Overridable Function SqlExec(ByVal sqlCommand As String) As Boolean
            Using conn = New SqliteConnection(ConnectionString)
                Try
                    conn.Open()

                    Logger.Log(sqlCommand, Category.Debug, Priority.None)

                    ' Execute the insert command
                    Dim cmd = New SqliteCommand(sqlCommand, conn)
                    cmd.ExecuteNonQuery()
                Catch ex As Exception
                    Throw New SqlException(ex.Message)
                End Try
            End Using
            Return True
        End Function


    End Class
End Namespace