﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Windows
Imports System.Windows.Controls
Imports Microsoft.Practices.Prism.Events
Imports Microsoft.Practices.Prism.Modularity
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Events
Imports MsdnDemo.Infrastructure.Portable.Base
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Logger
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.MvpVmBase
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.Phone.Events

Namespace MsdnDemo.Modules
    Public Class ApplicationControllerModule
        Inherits ModuleBase
        Implements IModule
#Region "Fields"
        Private ReadOnly _viewList As New List(Of ViewEventArgs)()

        Private ReadOnly _moduleRoles As New Dictionary(Of String, String())()

        Private Property DefaultView() As Object
#End Region

#Region "Properties "
        <Dependency()>
        Public Shadows Property Logger() As ILogger
            Get
                Return MyBase.Logger
            End Get
            Set(ByVal value As ILogger)
                MyBase.Logger = value
            End Set
        End Property

        <Dependency()>
        Public Property EventAggregator() As IEventAggregator

        <Dependency()>
        Public Property Container() As IUnityContainer
#End Region

        Public Shadows Sub Initialize() Implements IModule.Initialize
        End Sub
        ''' <summary>
        ''' Registers the types.
        ''' </summary>
        Protected Overrides Sub RegisterTypes()
            MyBase.RegisterTypes()

            EventAggregator.GetEvent(Of ModuleEvent)().Subscribe(AddressOf HandleModuleEvents, True)

            EventAggregator.GetEvent(Of ViewEvent)().Subscribe(AddressOf HandleViewEvents, True)
        End Sub

        ''' <summary>
        ''' Gets the type from the supplied name
        ''' </summary>
        ''' <param name="viewName">Name of the view.</param>
        ''' <returns></returns>
        Public Function GetTypeFromName(ByVal viewName As String) As Type
            Return _viewList.Where(Function(item) item.View.GetType().Name = viewName).Select(Function(item) item.View.GetType()).FirstOrDefault()
        End Function

        ''' <summary>
        ''' Activates the view.
        ''' </summary>
        ''' <param name="viewName">Name of the view.</param>
        Public Sub ActivateView(ByVal viewName As String)
            Logger.Log("ViewName = [{0}]", viewName)


            ' Activate this view (deactivates others to include Login)
            EventAggregator.GetEvent(Of ViewEvent)().Publish(New ViewEventArgs With {.ProcessType = ProcessType.ActivateView, .ViewTypeToActivate = GetTypeFromName(viewName)})
        End Sub

        ''' <summary>
        ''' Handles the view events.
        ''' </summary>
        ''' <param name="e">The <see cref="ViewEventArgs"/> 
        ''' instance containing the event data.</param>
        Public Sub HandleViewEvents(ByVal e As ViewEventArgs)
            Logger.Log("ProcessType:[{0}] View:[{1}] ViewTypeToActivate:{2}  ViewNameToActivate:{3}", e.ProcessType, e.View, e.ViewTypeToActivate, e.ViewNameToActivate)

            Select Case e.ProcessType
                ' REGISTER A VIEW 
                Case ProcessType.NewView
                    e.ViewTypeToActivate = e.View.GetType()
                    _viewList.Add(e)

                Case ProcessType.ActivateView
                    ' Activate by name (recursively)
                    If e.ViewTypeToActivate Is Nothing AndAlso e.ViewNameToActivate IsNot Nothing Then
                        Logger.Log("ViewNameToActivate = [{0}]", e.ViewNameToActivate)
                        ActivateView(e.ViewNameToActivate)
                        Return
                    End If

                    If e.ViewTypeToActivate Is Nothing Then
                        Logger.Log("ViewTypeToActivate was null - using default view")
                        e.ViewTypeToActivate = GetTypeFromName("SecurityView")
                    End If

                    Logger.Log("ViewTypeToActivate = [{0}]", e.ViewTypeToActivate)

                    ' Activate by type - ensure only one view is visible at any given time
                    For Each item In _viewList
                        Dim view = TryCast(item.View, UserControl)
                        view.Visibility = If(item.View.GetType().Name = e.ViewTypeToActivate.Name, Visibility.Visible, Visibility.Collapsed)

                        If view.Visibility = Visibility.Visible Then
                            Dim viewModel = TryCast(view.DataContext, IMvpVmViewModel)
                            If viewModel IsNot Nothing Then
                                viewModel.Presenter.ViewActivated(e)
                            End If

                        End If
                    Next item
            End Select
        End Sub

        ''' <summary>
        ''' Handles the module events. Loads modules on demand - as applicable
        ''' </summary>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.ModuleEventArgs"/> 
        ''' instance containing the event data.</param>
        Public Sub HandleModuleEvents(ByVal e As ModuleEventArgs)
            ' We're only interest in the login event
            If e.EventType <> ModuleEventType.Login Then
                Return
            End If

            ' Let's get the security view model so that we can
            ' examine the login states
            Dim securityViewModel = Container.Resolve(Of ISecurityViewModel)()

            ' Don't load any modules if the user did not authenticate
            ' Later, we can load a guest user screen if required
            If Not securityViewModel.IsAuthenticated Then
                Return
            End If

            ' Get module catalog so that we can get modules to load on-demand
            Dim moduleCatalog = Container.Resolve(Of IModuleCatalog)()

            ' We'll need the module manager to dynamically load the modules
            Dim moduleManager = Container.Resolve(Of IModuleManager)()

            ' If after processing this is still false then the user
            ' is not authorized to access module.  A message event
            ' will be raised below so that user can be notified
            Dim isAuthorized = False

            ' If not logged in then we won't have views to activate
            ' because they have not yet been loaded (on demand)
            If _moduleRoles.ContainsKey(e.ModuleName) Then
                ' Get the module roles
                Dim roles = _moduleRoles(e.ModuleName)
                If securityViewModel.IsInRole(roles) Then
                    ' If the user is authorized access to module activate it
                    ActivateView(e.PayLoad.ToString())
                    isAuthorized = True
                End If
            End If


            ' Iterate through each module and load all of the modules
            ' configured to be loaded on demand as it is requested
            For Each catalog In moduleCatalog.Modules
                If catalog.InitializationMode = InitializationMode.OnDemand Then
                    Dim roles() As String = {}
                    Dim moduleName = catalog.ModuleName
                    Dim roleOffset = catalog.ModuleName.IndexOf(":"c)
                    Dim isModuleAuthorized = False

                    ' If there are roles assigned to the module in the
                    ' GwnBootstrapper
                    If roleOffset > 0 Then
                        ' If roles are assigned to module name they will
                        ' be after a colon, e.g., MainModule:User,Admin
                        moduleName = catalog.ModuleName.Substring(0, roleOffset)

                        ' this isn't the module we are requesting so we won't
                        ' load it - only process requested module
                        If moduleName IsNot e.ModuleName Then
                            Continue For
                        End If

                        ' Split the module roles into the format expected by
                        ' the IsInRole method
                        roles = catalog.ModuleName.Substring(roleOffset + 1).Split(","c)

                        ' Is the user authorized to this module?
                        isModuleAuthorized = securityViewModel.IsInRole(roles)
                    End If

                    If isModuleAuthorized Then ' Only load module if authorized
                        ' Add it to our module roles collection so that we
                        ' can verify that future access to the module is 
                        ' authorized (user could login as someone else)
                        _moduleRoles.Add(moduleName, roles)

                        ' This will only load/launch one time - once loaded
                        ' the ActivateView() method above will be responsible
                        ' for activating the view.
                        catalog.ModuleName = moduleName
                        moduleManager.LoadModule(moduleName)

                        isAuthorized = True
                    End If
                End If
            Next catalog

            ' If not authorized then nothing will have happened - publish 
            ' message so that subscribers can notify user
            If Not isAuthorized Then
                EventAggregator.GetEvent(Of MessageEvent)().Publish(New MessageEventArgs With {.Message = "User not authorized access" & vbCrLf & "to this module!", .Sender = Me, .StatusCode = StatusCode.NotAuthorized})
            End If

        End Sub
    End Class
End Namespace