﻿#Const SqliteTest = True
Imports System.Collections.Generic
Imports Microsoft.Practices.Unity
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Entities
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.LayerBusinessLogic
Imports MsdnDemo.LayerBusinessLogic.Interfaces
Imports MsdnDemo.LayerDataAccess
Imports MsdnDemo.Mvc3.Tests.Base
Imports MsdnDemo.Mvc3.Tests.Mocks

Namespace MsdnDemo.Mvc3.Tests.Layers
    <TestClass()>
    Public Class UserFixture
        Inherits FixtureBase
        Private _testBll As IUserBll
        Private _mockBll As IMockBll

        <TestInitialize()>
        Public Sub InitializeTest()
            ' Registers default values (normally supplied by Prism)
            Initialize()

#If SqliteTest Then
            ' Roles
            ' User
            ' User Role
            ' Mock
            Container.RegisterType(Of IRolesBll, RoleBll)().RegisterType(Of IDal(Of RoleEntity), RolesDal)().RegisterType(Of IUserBll, UserBll)().RegisterType(Of IDal(Of UserEntity), UserDal)().RegisterType(Of IUserRoleBll, UserRoleBll)().RegisterType(Of IDal(Of UserRoleEntity), UserRoleDal)().RegisterType(Of IMockBll, MockBll)().RegisterType(Of IDal(Of MockEntity), MockDal)()
#Else
				' TODO: Sql server registrations go here....
#End If

            _testBll = Container.Resolve(Of IUserBll)()
            _testBll.Initialize()

            _mockBll = Container.Resolve(Of IMockBll)()
            _mockBll.Initialize()

        End Sub

        <TestMethod()>
        Public Sub CannotAddDuplicateUser()
            Const login As String = "NewUser@MyDomain.com"
            Const userRoles As String = "User,Guest"


            Dim userToCreate = New UserEntity With {.LastName = "User", .FirstName = "New", .Password = "MyPassword", .PrimaryEmail = login}

            ' Create the user with the role of User
            Dim status = _testBll.CreateUsersWithRoles(userToCreate, userRoles)

            Assert.AreEqual(StatusCode.Success, status)

            ' Create a duplicate user
            Dim args = New DalEventArgs With {.PrimaryKey = userToCreate.Id, .Data = userToCreate}
            status = _testBll.Create(args)

            Assert.AreEqual(StatusCode.UniquenessConstraint, status)
            Assert.AreEqual(args.PrimaryKey, userToCreate.Id)

            _testBll.DropTable()
        End Sub

        <TestMethod()>
        Public Sub CanDeleteRoleFromUser()
            Const login As String = "NewUser@MyDomain.com"
            Const userRoles As String = "User,Guest"

            Dim securityViewModel = Container.Resolve(Of ISecurityViewModel)()

            ' Create the user with the role of User
            _testBll.CreateUsersWithRoles(New UserEntity With {.LastName = "User", .FirstName = "New", .Password = "MyPassword", .PrimaryEmail = login}, userRoles)

            ' Set user login and authenticate
            securityViewModel.Login = login
            Assert.IsTrue(_testBll.AuthenticateUser("MyPassword"))

            Assert.IsTrue(securityViewModel.IsInRole("User"))
            Assert.IsTrue(securityViewModel.IsInRole("Guest"))
            Assert.IsFalse(securityViewModel.IsInRole("Admin"))

            ' Drop the guest role
            _testBll.DropRoleFromUser("Guest")

            ' Assert roles are correct
            Assert.IsTrue(securityViewModel.IsInRole("User"))
            Assert.IsFalse(securityViewModel.IsInRole("Guest"))
            Assert.IsFalse(securityViewModel.IsInRole("Admin"))

            _testBll.DropTable()
        End Sub

        <TestMethod()>
        Public Sub UserLoginHasRegisteredRoles()
            Dim securityViewModel = Container.Resolve(Of ISecurityViewModel)()

            securityViewModel.Login = "NewUser@MyDomain.com"
            Const password As String = "MyPassword" ' default password

            Dim status = _testBll.AuthenticateUser(password)
            Assert.IsFalse(status)

            Dim userRecord = New UserEntity With {.LastName = "User", .FirstName = "New", .Password = "MyPassword", .PrimaryEmail = "NewUser@MyDomain.com"}
            Dim roles = New List(Of RoleEntity) From {
             New RoleEntity With {.Role = "User"}}

            ' Create the user with the role of User
            _testBll.CreateUsersWithRoles(userRecord, roles)

            ' Set the security view model login
            securityViewModel.Login = userRecord.PrimaryEmail

            ' authenticate user - the security view model
            ' will hold authentication results
            _testBll.AuthenticateUser("MyPassword")

            Assert.AreEqual("NewUser@MyDomain.com", securityViewModel.Login)
            Assert.IsTrue(securityViewModel.IsAuthenticated)

            ' Only the User roles should be available
            Assert.IsTrue(securityViewModel.IsInRole("User"))
            Assert.IsFalse(securityViewModel.IsInRole("Admin"))
            Assert.IsFalse(securityViewModel.IsInRole("Guest"))

            Dim args = New DalEventArgs With {.PrimaryKey = securityViewModel.Id}

            ' Read record for the specified primary key (above)
            ' so that we can ensure it was created successfully
            Assert.AreEqual(StatusCode.Success, _testBll.Read(args))

            ' Get the user results from the args 
            Dim createdUser = args.GetResults(Of UserEntity)()

            ' Assert user record was created 
            Assert.IsNotNull(createdUser)
            Assert.AreEqual("New", createdUser.FirstName)
            Assert.AreEqual("User", createdUser.LastName)
            Assert.AreEqual("NewUser@MyDomain.com", createdUser.PrimaryEmail)
            Assert.AreEqual("MyPassword", createdUser.Password)

            _testBll.DropTable()
        End Sub


        <TestMethod()>
        Public Sub EnsureInvalidLoginFails()
            Dim securityViewModel = Container.Resolve(Of ISecurityViewModel)()
            securityViewModel.Login = "admin@mydomain.net"
            Const password As String = "Invalid password"

            Dim status = _testBll.AuthenticateUser(password)
            Assert.IsFalse(status)

            _testBll.DropTable()
        End Sub


        <TestMethod()>
        Public Sub AdminLoginHasDefaultRoles()
            Dim securityViewModel = Container.Resolve(Of ISecurityViewModel)()
            securityViewModel.Login = "admin@mydomain.net"
            Const password As String = "password" ' default password

            Dim status = _testBll.AuthenticateUser(password)
            Assert.IsTrue(status)
            Assert.IsTrue(securityViewModel.IsInRole("Admin"))
            Assert.IsTrue(securityViewModel.IsInRole("User"))
            Assert.IsFalse(securityViewModel.IsInRole("Guest"))

            _testBll.DropTable()
        End Sub


        <TestMethod()>
        Public Sub EmptyUserInfoTableCreatesDefaults()
            Dim resultArgs = New DalEventArgs

            Dim status = _testBll.List(resultArgs)
            Assert.AreEqual(StatusCode.Success, status)

            Dim list = New List(Of UserEntity)(resultArgs.GetResults(Of IEnumerable(Of UserEntity))())

            Assert.IsTrue(list.Count > 0, "Default UserInfo should have been created")

            ' Cleanup
            _testBll.DropTable()
        End Sub
    End Class
End Namespace